/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.system.service.impl;


import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.AllArgsConstructor;
import org.springblade.common.cache.DictBizCache;
import org.springblade.common.cache.SysCache;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.constant.TenantConstant;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tenant.BladeTenantProperties;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.jackson.JsonUtil;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.*;
import org.springblade.modules.auth.enums.UserEnum;
import org.springblade.modules.fjInfo.entity.FjInfoEntity;
import org.springblade.modules.rk.vo.LbVO;
import org.springblade.modules.system.entity.*;
import org.springblade.modules.system.excel.FjxxExcel;
import org.springblade.modules.system.excel.UserExcel;
import org.springblade.modules.system.excel.UserYhExcel;
import org.springblade.modules.system.mapper.UserMapper;
import org.springblade.modules.system.service.*;
import org.springblade.modules.system.vo.*;
import org.springblade.modules.system.wrapper.UserWrapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * 服务实现类
 *
 * @author Chill
 */
@Service
@AllArgsConstructor
public class UserServiceImpl extends BaseServiceImpl<UserMapper, User> implements IUserService {
	private static final String GUEST_NAME = "guest";

	private final IUserDeptService userDeptService;
	private final IUserOauthService userOauthService;
	private final IRoleService roleService;
	private final BladeTenantProperties tenantProperties;
	private final IDeptService deptService;
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean submit(User user) {
		if (StringUtil.isBlank(user.getTenantId())) {
			user.setTenantId(BladeConstant.ADMIN_TENANT_ID);
		}
		String tenantId = user.getTenantId();
		Tenant tenant = SysCache.getTenant(tenantId);
		if (Func.isNotEmpty(tenant)) {
			Integer accountNumber = tenant.getAccountNumber();
			if (tenantProperties.getLicense()) {
				String licenseKey = tenant.getLicenseKey();
				String decrypt = DesUtil.decryptFormHex(licenseKey, TenantConstant.DES_KEY);
				accountNumber = JsonUtil.parse(decrypt, Tenant.class).getAccountNumber();
			}
			Long tenantCount = baseMapper.selectCount(Wrappers.<User>query().lambda().eq(User::getTenantId, tenantId));
			if (accountNumber != null && accountNumber > 0 && accountNumber <= tenantCount) {
				throw new ServiceException("当前租户已到最大账号额度!");
			}
		}
		if (Func.isNotEmpty(user.getPassword())) {
			user.setPassword(DigestUtil.encrypt(user.getPassword()));
		}
		Long userCount = baseMapper.selectCount(Wrappers.<User>query().lambda().eq(User::getTenantId, tenantId).eq(User::getAccount, user.getAccount()));
		if (userCount > 0L && Func.isEmpty(user.getId())) {
			throw new ServiceException(StringUtil.format("当前用户 [{}] 已存在!", user.getAccount()));
		}
		return save(user) && submitUserDept(user);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean updateUser(User user) {
		String tenantId = user.getTenantId();
		Long userCount = baseMapper.selectCount(
			Wrappers.<User>query().lambda()
				.eq(User::getTenantId, tenantId)
				.eq(User::getAccount, user.getAccount())
				.notIn(User::getId, user.getId())
		);
		if (userCount > 0L) {
			throw new ServiceException(StringUtil.format("当前用户 [{}] 已存在!", user.getAccount()));
		}
		return updateUserInfo(user) && submitUserDept(user);
	}

	@Override
	public boolean updateUserInfo(User user) {
		user.setPassword(null);
		return updateById(user);
	}

	private boolean submitUserDept(User user) {
		List<Long> deptIdList = Func.toLongList(user.getDeptId());
		List<UserDept> userDeptList = new ArrayList<>();
		deptIdList.forEach(deptId -> {
			UserDept userDept = new UserDept();
			userDept.setUserId(user.getId());
			userDept.setDeptId(deptId);
			userDeptList.add(userDept);
		});
		userDeptService.remove(Wrappers.<UserDept>update().lambda().eq(UserDept::getUserId, user.getId()));
		return userDeptService.saveBatch(userDeptList);
	}

	@Override
	public IPage<User> selectUserPage(IPage<User> page, User user, String deptId, String tenantId) {
		/*List<Long> deptIdList = new ArrayList<>();
		if (!deptId.toString().equals("220000000000")) {
			deptIdList = SysCache.getDeptChildIds(deptId);
		}*/
		return page.setRecords(baseMapper.selectUserPage(page, deptId, user, tenantId));
	}
	@Override
	public IPage<User> selectUserPage1(IPage<User> page, User user, String deptId, String tenantId) {
		//List<Long> deptIdList = SysCache.getDeptChildIds(deptId);
		return page.setRecords(baseMapper.selectUserPage(page, deptId, user, tenantId));
	}

	@Override
	public IPage<UserVO> selectUserSearch(UserVO user, Query query) {
		LambdaQueryWrapper<User> queryWrapper = Wrappers.<User>query().lambda();
		String tenantId = AuthUtil.getTenantId();
		if (StringUtil.isNotBlank(tenantId)) {
			queryWrapper.eq(User::getTenantId, tenantId);
		}
		if (StringUtil.isNotBlank(user.getName())) {
			queryWrapper.like(User::getName, user.getName());
		}
		if (StringUtil.isNotBlank(user.getDeptName())) {
			String deptIds = SysCache.getDeptIdsByFuzzy(AuthUtil.getTenantId(), user.getDeptName());
			if (StringUtil.isNotBlank(deptIds)) {
				queryWrapper.and(wrapper -> {
					List<String> ids = Func.toStrList(deptIds);
					ids.forEach(id -> wrapper.like(User::getDeptId, id).or());
				});
			}
		}
		if (StringUtil.isNotBlank(user.getPostName())) {
			String postIds = SysCache.getPostIdsByFuzzy(AuthUtil.getTenantId(), user.getPostName());
			if (StringUtil.isNotBlank(postIds)) {
				queryWrapper.and(wrapper -> {
					List<String> ids = Func.toStrList(postIds);
					ids.forEach(id -> wrapper.like(User::getPostId, id).or());
				});
			}
		}
		IPage<User> pages = this.page(Condition.getPage(query), queryWrapper);
		return UserWrapper.build().pageVO(pages);
	}

	@Override
	public User userByAccount(String tenantId, String account) {
		return baseMapper.selectOne(Wrappers.<User>query().lambda().eq(User::getTenantId, tenantId).eq(User::getAccount, account).eq(User::getIsDeleted, BladeConstant.DB_NOT_DELETED));
	}

	@Override
	public UserInfo userInfo(Long userId) {
		User user = baseMapper.selectById(userId);
		return buildUserInfo(user);
	}

	@Override
	public UserInfo userInfo(String tenantId, String account, String password) {
		User user = baseMapper.getUser(tenantId, account, password);
		return buildUserInfo(user);
	}

	@Override
	public UserInfo userInfo(String tenantId, String account, String password, UserEnum userEnum) {
		User user = baseMapper.getUser(tenantId, account, password);
		// ldx写的
		if (null != user) {
			//登录返回用户postid对应的部门
			String postId = user.getPostId();
			if(StringUtils.isNotBlank(postId)){
				String[] split = postId.split(",");
				Dept userPostDept = deptService.getById(Long.parseLong(split[0]));
				if(userPostDept!=null){
					user.setBmMc(userPostDept.getDeptName());
				}
			}
			if(StringUtils.isNotBlank(user.getRoleId()) && (DictBizCache.getValue("jsjh",1)).contains(user.getRoleId())){
				if(StringUtils.isNotBlank(user.getDeptId())){
					String deptId = user.getDeptId();
					String[] split = deptId.split(",");
					Dept one = deptService.getOne(Wrappers.<Dept>lambdaQuery().eq(Dept::getId, Long.parseLong(split[0])));
					if(one != null){
						if(one.getDeptCategory() == 7){
							user.setPcsDm(one.getParentId().toString());
						}
					}
				}
			}
		}
		return buildUserInfo(user, userEnum);
	}

	private UserInfo buildUserInfo(User user) {
		return buildUserInfo(user, UserEnum.WEB);
	}

	private UserInfo buildUserInfo(User user, UserEnum userEnum) {
		if (ObjectUtil.isEmpty(user)) {
			return null;
		}
		UserInfo userInfo = new UserInfo();
		userInfo.setUser(user);
		if (Func.isNotEmpty(user)) {
			List<String> roleAlias = roleService.getRoleAliases(user.getRoleId());
			userInfo.setRoles(roleAlias);
		}
		// 根据每个用户平台，建立对应的detail表，通过查询将结果集写入到detail字段
		Kv detail = Kv.create().set("type", userEnum.getName());
		if (userEnum == UserEnum.WEB) {
			UserWeb userWeb = new UserWeb();
			UserWeb query = userWeb.selectOne(Wrappers.<UserWeb>lambdaQuery().eq(UserWeb::getUserId, user.getId()));
			if (ObjectUtil.isNotEmpty(query)) {
				detail.set("ext", query.getUserExt());
			}
		} else if (userEnum == UserEnum.APP) {
			UserApp userApp = new UserApp();
			UserApp query = userApp.selectOne(Wrappers.<UserApp>lambdaQuery().eq(UserApp::getUserId, user.getId()));
			if (ObjectUtil.isNotEmpty(query)) {
				detail.set("ext", query.getUserExt());
			}
		} else {
			UserOther userOther = new UserOther();
			UserOther query = userOther.selectOne(Wrappers.<UserOther>lambdaQuery().eq(UserOther::getUserId, user.getId()));
			if (ObjectUtil.isNotEmpty(query)) {
				detail.set("ext", query.getUserExt());
			}
		}
		userInfo.setDetail(detail);
		return userInfo;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public UserInfo userInfo(UserOauth userOauth) {
		UserOauth uo = userOauthService.getOne(Wrappers.<UserOauth>query().lambda().eq(UserOauth::getUuid, userOauth.getUuid()).eq(UserOauth::getSource, userOauth.getSource()));
		UserInfo userInfo;
		if (Func.isNotEmpty(uo) && Func.isNotEmpty(uo.getUserId())) {
			userInfo = this.userInfo(uo.getUserId());
			userInfo.setOauthId(Func.toStr(uo.getId()));
		} else {
			userInfo = new UserInfo();
			if (Func.isEmpty(uo)) {
				userOauthService.save(userOauth);
				userInfo.setOauthId(Func.toStr(userOauth.getId()));
			} else {
				userInfo.setOauthId(Func.toStr(uo.getId()));
			}
			User user = new User();
			user.setAccount(userOauth.getUsername());
			user.setTenantId(userOauth.getTenantId());
			userInfo.setUser(user);
			userInfo.setRoles(Collections.singletonList(GUEST_NAME));
		}
		return userInfo;
	}

	@Override
	public boolean grant(String userIds, String roleIds) {
		User user = new User();
		user.setRoleId(roleIds);
		return this.update(user, Wrappers.<User>update().lambda().in(User::getId, Func.toLongList(userIds)));
	}

	@Override
	public boolean resetPassword(String userIds) {
		User user = new User();
		user.setPassword(DigestUtil.encrypt(CommonConstant.DEFAULT_PASSWORD));
		user.setUpdateTime(DateUtil.now());
		return this.update(user, Wrappers.<User>update().lambda().in(User::getId, Func.toLongList(userIds)));
	}

	@Override
	public boolean updatePassword(Long userId, String oldPassword, String newPassword, String newPassword1) {
		User user = getById(userId);
		if (!newPassword.equals(newPassword1)) {
			throw new ServiceException("请输入正确的确认密码!");
		}
		if (!user.getPassword().equals(DigestUtil.hex(oldPassword))) {
			throw new ServiceException("原密码不正确!");
		}
		return this.update(Wrappers.<User>update().lambda().set(User::getPassword, DigestUtil.hex(newPassword)).eq(User::getId, userId));
	}

	@Override
	public boolean removeUser(String userIds) {
		if (Func.contains(Func.toLongArray(userIds), AuthUtil.getUserId())) {
			throw new ServiceException("不能删除本账号!");
		}
		return deleteLogic(Func.toLongList(userIds));
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void importUser(List<UserExcel> data, Boolean isCovered) {
//		data.forEach(userExcel -> {
//			User user = Objects.requireNonNull(BeanUtil.copy(userExcel, User.class));
//			// 设置用户平台
//			user.setUserType(Func.toInt(DictCache.getKey(DictEnum.USER_TYPE, userExcel.getUserTypeName()), 1));
//			// 设置部门ID
//			user.setDeptId(Func.toStrWithEmpty(SysCache.getDeptIds(userExcel.getTenantId(), userExcel.getDeptName()), StringPool.EMPTY));
//			// 设置岗位ID
//			user.setPostId(Func.toStrWithEmpty(SysCache.getPostIds(userExcel.getTenantId(), userExcel.getPostName()), StringPool.EMPTY));
//			// 设置角色ID
//			user.setRoleId(Func.toStrWithEmpty(SysCache.getRoleIds(userExcel.getTenantId(), userExcel.getRoleName()), StringPool.EMPTY));
//			// 设置租户ID
//			if (!AuthUtil.isAdministrator() || StringUtil.isBlank(user.getTenantId())) {
//				user.setTenantId(AuthUtil.getTenantId());
//			}
//			// 覆盖数据
//			if (isCovered) {
//				// 查询用户是否存在
//				User oldUser = UserCache.getUser(userExcel.getTenantId(), userExcel.getAccount());
//				if (oldUser != null && oldUser.getId() != null) {
//					user.setId(oldUser.getId());
//					this.updateUser(user);
//					return;
//				}
//			}
//			// 获取默认密码配置
//			String initPassword = ParamCache.getValue(DEFAULT_PARAM_PASSWORD);
//			user.setPassword(initPassword);
//			this.submit(user);
//		});
	}

	@Override
	public List<UserExcel> exportUser(Wrapper<User> queryWrapper) {
//		List<UserExcel> userList = baseMapper.exportUser(queryWrapper);
//		userList.forEach(user -> {
//			user.setUserTypeName(DictCache.getValue(DictEnum.USER_TYPE, user.getUserType()));
//			user.setRoleName(StringUtil.join(SysCache.getRoleNames(user.getRoleId())));
//			user.setDeptName(StringUtil.join(SysCache.getDeptNames(user.getDeptId())));
//			user.setPostName(StringUtil.join(SysCache.getPostNames(user.getPostId())));
//		});
//		return userList;
		return null;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean registerGuest(User user, Long oauthId) {
		Tenant tenant = SysCache.getTenant(user.getTenantId());
		if (tenant == null || tenant.getId() == null) {
			throw new ServiceException("租户信息错误!");
		}
		UserOauth userOauth = userOauthService.getById(oauthId);
		if (userOauth == null || userOauth.getId() == null) {
			throw new ServiceException("第三方登陆信息错误!");
		}
		user.setRealName(user.getName());
		user.setAvatar(userOauth.getAvatar());
		user.setRoleId(StringPool.MINUS_ONE);
		user.setDeptId(StringPool.MINUS_ONE);
		user.setPostId(StringPool.MINUS_ONE);
		boolean userTemp = this.submit(user);
		userOauth.setUserId(user.getId());
		userOauth.setTenantId(user.getTenantId());
		boolean oauthTemp = userOauthService.updateById(userOauth);
		return (userTemp && oauthTemp);
	}

	@Override
	public boolean updatePlatform(Long userId, Integer userType, String userExt) {
		if (userType.equals(UserEnum.WEB.getCategory())) {
			UserWeb userWeb = new UserWeb();
			UserWeb query = userWeb.selectOne(Wrappers.<UserWeb>lambdaQuery().eq(UserWeb::getUserId, userId));
			if (ObjectUtil.isNotEmpty(query)) {
				userWeb.setId(query.getId());
			}
			userWeb.setUserId(userId);
			userWeb.setUserExt(userExt);
			return userWeb.insertOrUpdate();
		} else if (userType.equals(UserEnum.APP.getCategory())) {
			UserApp userApp = new UserApp();
			UserApp query = userApp.selectOne(Wrappers.<UserApp>lambdaQuery().eq(UserApp::getUserId, userId));
			if (ObjectUtil.isNotEmpty(query)) {
				userApp.setId(query.getId());
			}
			userApp.setUserId(userId);
			userApp.setUserExt(userExt);
			return userApp.insertOrUpdate();
		} else {
			UserOther userOther = new UserOther();
			UserOther query = userOther.selectOne(Wrappers.<UserOther>lambdaQuery().eq(UserOther::getUserId, userId));
			if (ObjectUtil.isNotEmpty(query)) {
				userOther.setId(query.getId());
			}
			userOther.setUserId(userId);
			userOther.setUserExt(userExt);
			return userOther.insertOrUpdate();
		}
	}

	@Override
	public UserVO platformDetail(User user) {
		User detail = baseMapper.selectOne(Condition.getQueryWrapper(user));
		UserVO userVO = UserWrapper.build().entityVO(detail);
		if (userVO.getUserType().equals(UserEnum.WEB.getCategory())) {
			UserWeb userWeb = new UserWeb();
			UserWeb query = userWeb.selectOne(Wrappers.<UserWeb>lambdaQuery().eq(UserWeb::getUserId, user.getId()));
			if (ObjectUtil.isNotEmpty(query)) {
				userVO.setUserExt(query.getUserExt());
			}
		} else if (userVO.getUserType().equals(UserEnum.APP.getCategory())) {
			UserApp userApp = new UserApp();
			UserApp query = userApp.selectOne(Wrappers.<UserApp>lambdaQuery().eq(UserApp::getUserId, user.getId()));
			if (ObjectUtil.isNotEmpty(query)) {
				userVO.setUserExt(query.getUserExt());
			}
		} else {
			UserOther userOther = new UserOther();
			UserOther query = userOther.selectOne(Wrappers.<UserOther>lambdaQuery().eq(UserOther::getUserId, user.getId()));
			if (ObjectUtil.isNotEmpty(query)) {
				userVO.setUserExt(query.getUserExt());
			}
		}
		return userVO;
	}

	@Override
	public String getCodeByUsername(String username) {
		return baseMapper.getCodeByUsername(username);
	}

	@Override
	public String getUsernameByCode(String code) {
		return baseMapper.getUsernameByCode(code);
	}

	@Override
	public void updateCode(String username, String code) {
		baseMapper.updateCode(username, code);
	}

	@Override
	public User selectUserIdByPhone(String xpmjlxdh) {
		return baseMapper.selectUserIdByPhone(xpmjlxdh);
	}

	@Override
	public User selectUser(Long userId) {
		return baseMapper.selectUser(userId);
	}

	@Override
	public List<LbVO> selectSq(Long userId) {
		return baseMapper.selectSq(userId);
	}

	@Override
	public List<UserVO> getMjList(Map<String, Object> params) {
		return baseMapper.getMjList(params);
	}

	@Override
	public String getUsernameBySfzh(String sfzh) {
		return baseMapper.getUsernameBySfzh(sfzh);
	}

	@Override
	public String getRoleId(String roleName) {
		return baseMapper.getRoleId(roleName);
	}

	@Override
	public String getSqId(String sqName, String pcsName, Long fjId) {
		return baseMapper.getSqId(sqName, pcsName, fjId);
	}

	@Override
	public String getPcsId(String pcsName, Long fjId) {
		return baseMapper.getPcsId(pcsName, fjId);
	}

	@Override
	public List<User> getUserList() {
		return baseMapper.getUserList();
	}

	@Override
	public void updateSfzh(User user) {
		baseMapper.updateSfzh(user);
	}

	@Override
	public String getRoleName(String username) {
		return baseMapper.getRoleName(username);
	}

	@Override
	public User getById(Long userId) {
		return baseMapper.getById(userId);
	}

	@Override
	public void updateLastLoginTime(String username) {
		baseMapper.updateLastLoginTime(username);
	}

	@Override
	public String getZxxdSqId(String pcsDm) {
		return baseMapper.getZxxdSqId(pcsDm);
	}

	@Override
	public Long getIdByAccount(String account) {
		return baseMapper.getIdByAccount(account);
	}

	@Override
	public User getUserNameById(Long createUser) {
		return baseMapper.getUserNameById(createUser);
	}

	@Override
	public boolean updateDeptId(Long userId, String ids) {
		return baseMapper.updateDeptId(userId, ids);
	}

	@Override
	public String getPcsIdByPcsName(String pcsName, int deptCategory) {
		return baseMapper.getPcsIdByPcsName(pcsName, deptCategory);
	}

	@Override
	public SyBottomVO getSyBottom(String deptId) {
		return baseMapper.getSyBottom(deptId);
	}

	@Override
	public SyCenterVO getSyCenter(String deptId) {
		SyCenterVO syCenterVO = new SyCenterVO();
		SyCenterVO fwinfo = baseMapper.getSyCenterFwInfo(deptId);
		SyCenterVO fwry = baseMapper.getSyCenterFwry(deptId);
		SyCenterVO jxcs = baseMapper.getSyCenterJxcs(deptId);
		SyCenterVO csjc = baseMapper.getSyCenterCsjc(deptId);
		SyCenterVO mdjf = baseMapper.getSyCenterMdjf(deptId);
		SyCenterVO yjjy = baseMapper.getSyCenterYjjy(deptId);
		syCenterVO.setFwinfo(fwinfo.getFwinfo());
		syCenterVO.setFwry(fwry.getFwry());
		syCenterVO.setJxcs(jxcs.getJxcs());
		syCenterVO.setCsjc(csjc.getCsjc());
		syCenterVO.setMdjf(mdjf.getMdjf());
		syCenterVO.setYjjy(yjjy.getYjjy());
		return syCenterVO;
	}

	@Override
	public SyCenterVO getSyCenterzs() {
		SyCenterVO syCenterVO = new SyCenterVO();
		SyCenterVO fwinfozs = baseMapper.getSyCenterFwInfoZs();
		SyCenterVO fwryzs = baseMapper.getSyCenterFwryZs();
		SyCenterVO jxcszs = baseMapper.getSyCenterJxcsZs();
		SyCenterVO csjczs = baseMapper.getSyCenterCsjcZs();
		SyCenterVO mdjfzs = baseMapper.getSyCenterMdjfZs();
		SyCenterVO yjjyzs = baseMapper.getSyCenterYjjyZs();
		syCenterVO.setFwinfozs(fwinfozs.getFwinfozs());
		syCenterVO.setFwryzs(fwryzs.getFwryzs());
		syCenterVO.setJxcszs(jxcszs.getJxcszs());
		syCenterVO.setCsjczs(csjczs.getCsjczs());
		syCenterVO.setMdjfzs(mdjfzs.getMdjfzs());
		syCenterVO.setYjjyzs(yjjyzs.getYjjyzs());
		return syCenterVO;
	}

	@Override
	public SyTopVO getSyTop(String dept) {
		return baseMapper.getSyTop(dept);
	}

	@Override
	public User getByName(String cjr) {
		return baseMapper.getByName(cjr);
	}

	@Override
	public User selectAccountByUserName(String username) {
		return baseMapper.selectAccountByUserName(username);
	}

	@Override
	public PhVO getSyCenterPh(String deptId, String dept) {
		return baseMapper.getSyCenterPh(deptId, dept);
	}

	@Override
	public List<GsdrsjbVO> getGsdrsjb(GsdrsjbVO gsdrsjbVO) {
		return baseMapper.getGsdrsjb(gsdrsjbVO);
	}

	@Override
	public List<UserYhExcel> getYhByCreateTime(User user) {
		List<Long> deptIdList = SysCache.getDeptChildIds(Long.parseLong(user.getDeptId()));
		return baseMapper.getYhByCreateTime(user, deptIdList);
	}

	@Override
	public boolean qcCode(String id) {
		return baseMapper.qcCode(id);
	}

	@Override
	public String getPhone(Long id) {
		return baseMapper.getPhone(id);
	}

	@Override
	public Integer getPhoneXz(String phone) {
		return baseMapper.getPhoneXz(phone);
	}

	@Override
	public int getFjsl(Long id) {
		return baseMapper.getFjsl(id);
	}

	@Override
	public IPage<User> getFjgl(IPage<Object> page, User user) {
		return baseMapper.getFjgl(page, user);
	}

	@Override
	public List<ZdVO> getPcsFj(Dept dept) {
		return baseMapper.getPcsFj(dept);
	}

	@Override
	public boolean updateFj(FjInfoEntity fjInfo, Long mjid) {
		return baseMapper.updateFj(fjInfo, mjid);
	}

	@Override
	public boolean updateUserFj(User user) {
		return baseMapper.updateUserFj(user);
	}

	@Override
	public List<FjInfoEntity> getFj(Long id) {
		return baseMapper.getFj(id);
	}

	@Override
	public boolean deleteFj(Long id) {
		return baseMapper.deleteFj(id);
	}

	@Override
	public List<FjxxExcel> exportFjxx(Map<String, Object> params) {
		return baseMapper.exportFjxx(params);
	}

	@Override
	public User selectMj(Long id) {
		return baseMapper.selectMj(id);
	}

	@Override
	public User selectMjPcs(Long id) {
		return baseMapper.selectMjPcs(id);
	}

	@Override
	public void updateStatus() {
		baseMapper.updateStatus();
	}

	@Override
	public void zx(User user) {
		baseMapper.zx(user);
	}

	@Override
	public Integer getFwzs(String deptId) {
		return baseMapper.getFwzs(deptId);
	}

	@Override
	public Integer getSydwzs(String deptId) {
		return baseMapper.getSydwzs(deptId);
	}

	@Override
	public Integer getCzrk(String deptId) {
		return baseMapper.getCzrk(deptId);
	}

	@Override
	public Integer getJzrk(String deptId) {
		return baseMapper.getJzrk(deptId);
	}

	@Override
	public Integer getZzrk(String deptId) {
		return baseMapper.getZzrk(deptId);
	}

	@Override
	public Integer getJwrk(String deptId) {
		return baseMapper.getJwrk(deptId);
	}

	@Override
	public Integer getWlrk(String deptId) {
		return baseMapper.getWlrk(deptId);
	}

	@Override
	public Integer getSwzdrzs(String deptId) {
		return baseMapper.getSwzdrzs(deptId);
	}

	@Override
	public Integer getTddxzs(String deptId) {
		return baseMapper.getTddxzs(deptId);
	}

	@Override
	public Integer getMdjfzs(String deptId) {
		return baseMapper.getMdjfzs(deptId);
	}

	@Override
	public Integer getJxcszs(String deptId) {
		return baseMapper.getJxcszs(deptId);
	}

	@Override
	public RkxxEntity selectRyxxBySfzh(String sfzh) {
		return baseMapper.selectRyxxBySfzh(sfzh);
	}
}
